use std::collections::HashMap;
use std::error::Error;

use ldap3::{LdapConn, Scope, SearchEntry};
use yaml_rust::Yaml;

// create an LDAP connection object
pub fn connect(ldap_connection_info: HashMap<&str, &str>) -> LdapConn {
  let conn_string = format!("{}://{}:{}",
    ldap_connection_info.get("protocol").unwrap(),
    ldap_connection_info.get("server").unwrap(),
    ldap_connection_info.get("port").unwrap());
  let ldap_conn = LdapConn::new(&conn_string);

  if ldap_conn.is_ok() {
    return ldap_conn.ok().unwrap();
  } else {
    panic!("{:?}", ldap_conn.err().unwrap());
  }
}

// perform an LDAP search
pub fn do_search(ldap_conn: LdapConn, search_base: &str, search_filter: &str) -> Result<(), Box<Error>> {
  let (rs, _res) = ldap_conn.search(
    search_base,
    Scope::Subtree,
    search_filter,
    vec!["*"]
  )?.success()?;

  for entry in rs {
    let search_entry = SearchEntry::construct(entry);
    // print the entry's DN
    println!("dn: {}", search_entry.dn);

    // loop through the entry's attributes and print them out
    for (key, val) in search_entry.attrs {
      println!("{}: {:?}", key, val);
    }

    // print a blank line
    println!("");
  }

  Ok(())
}

// perform an add user operation
pub fn do_add_user(ldap_conn: LdapConn, user_conf: Yaml, user_info: &HashMap<&str, &str>) -> Result<(), Box<Error>> {
  let user_dn = format!("{}={},{}", user_conf["id_attr"].as_str().unwrap(), user_info.get("username").unwrap(), user_conf["base"].as_str().unwrap());
  let user_home = format!("/nfs/users/{}/home", user_info.get("username").unwrap());
  let user_display = format!("{} {}", user_info.get("first-name").unwrap(), user_info.get("last-name").unwrap());

  let user_vec = vec![
    (user_conf["id_attr"].as_str().unwrap(), hashset!{user_info.get("username").unwrap().to_owned()}),
    ("mail", hashset!{user_info.get("username").unwrap().to_owned()}),
    ("displayName", hashset!{user_display.as_str()}),
    ("givenName", hashset!{user_info.get("first-name").unwrap().to_owned()}),
    ("sn", hashset!{user_info.get("last-name").unwrap().to_owned()}),
    ("uidNumber", hashset!{user_info.get("id-number").unwrap().to_owned()}),
    ("gidNumber", hashset!{user_info.get("id-number").unwrap().to_owned()}),
    ("homeDirectory", hashset!{user_home.as_str()}),
    ("objectClass", hashset!{user_conf["attrs"]["objectClass"].as_vec().unwrap()})
  ];

  let res = ldap_conn.add(&user_dn, user_vec)?.success()?;
  println!("{:?}", res);

  Ok(())
}
