extern crate clap;
extern crate ldap3;
#[macro_use]
extern crate maplit;
extern crate yaml_rust;

mod ldap;

use std::collections::HashMap;
use std::fs::File;
use std::io::prelude::*;

use clap::{App, Arg, SubCommand};
use yaml_rust::YamlLoader;

fn main() {
  // read user configuration file
  let yaml_file = File::open("/home/gballan/.muldap.yaml");
  let mut yaml_raw = String::new();
  yaml_file.unwrap().read_to_string(&mut yaml_raw);
  // parse user configuration
  let yaml_multi = YamlLoader::load_from_str(&yaml_raw).unwrap();
  let yaml = &yaml_multi[0];

  // parse options
  let matches = App::new("Metaunix.net LDAP tool")
    .version("0.1.0")
    .author("Gregory W. Ballantine <gballantine555@gmail.com>")
    .about("Tool to manage LDAP resources for Metaunix.net")

    // connection protocol argument
    .arg(Arg::with_name("protocol")
      .short("P")
      .long("protocol")
      .takes_value(true)
      .required(true)
      .default_value(yaml["muldap"]["protocol"].as_str().unwrap_or("ldap"))
      .help("LDAP connection protocol to use (typically ldap or ldaps)"))
    // server argument
    .arg(Arg::with_name("server")
      .short("s")
      .long("server")
      .takes_value(true)
      .required(true)
      .default_value(yaml["muldap"]["server"].as_str().unwrap())
      .help("LDAP server to search"))
    // server port argument
    .arg(Arg::with_name("port")
      .short("p")
      .long("port")
      .takes_value(true)
      .required(true)
      .default_value(yaml["muldap"]["port"].as_str().unwrap_or("389"))
      .help("Port on which the LDAP server is listening on"))

    // LDAP search subcommand
    .subcommand(SubCommand::with_name("search")
      .about("Search an LDAP directory")
      // search base argument
      .arg(Arg::with_name("base")
        .short("b")
        .long("base")
        .takes_value(true)
        .required(true)
        .default_value(yaml["muldap"]["search_base"].as_str().unwrap())
        .help("Base DN to search with"))
      // search filter argument
      .arg(Arg::with_name("filter")
        .short("f")
        .long("filter")
        .takes_value(true)
        .required(false)
        .default_value("(&(objectClass=*))")
        .help("LDAP search filter")))

    // LDAP add resource subcommand
    .subcommand(SubCommand::with_name("add")
      .about("Add an LDAP entry")

      // add user entry subcommand
      .subcommand(SubCommand::with_name("user")
        .about("Add an LDAP user entry")
        // Username argument
        .arg(Arg::with_name("username")
          .short("u")
          .long("username")
          .takes_value(true)
          .required(true)
          .help("User's login username"))
        // Email address argument
        .arg(Arg::with_name("email")
          .short("e")
          .long("email")
          .takes_value(true)
          .required(true)
          .help("User's email address"))
        // First name argument
        .arg(Arg::with_name("first-name")
          .short("f")
          .long("first-name")
          .takes_value(true)
          .required(true)
          .help("User's first name"))
        // Last name argument
        .arg(Arg::with_name("last-name")
          .short("l")
          .long("last-name")
          .takes_value(true)
          .required(true)
          .help("User's last name"))
        // ID number argument
        .arg(Arg::with_name("id-number")
          .short("i")
          .long("id-number")
          .takes_value(true)
          .required(true)
          .help("User's ID number"))))

    .get_matches();

  // create a new hashmap for the connection info
  let mut ldap_connection_info = HashMap::new();
  // add connection info to hashmap
  ldap_connection_info.insert("protocol", matches.value_of("protocol").unwrap());
  ldap_connection_info.insert("server", matches.value_of("server").unwrap());
  ldap_connection_info.insert("port", matches.value_of("port").unwrap());

  // create LDAP connection
  let ldap_conn = ldap::connect(ldap_connection_info);

  // search subcommand
  if let Some(matches) = matches.subcommand_matches("search") {
    // get search options
    let search_base = matches.value_of("base").unwrap();
    let search_filter = matches.value_of("filter").unwrap();

    // run the LDAP search
    match ldap::do_search(ldap_conn, search_base, search_filter) {
      Ok(_) => {},
      Err(e) => println!("{:?}", e),
    }
  } else if let Some(matches) = matches.subcommand_matches("add") {
    // perform an LDAP bind
    let bind_dn = yaml["muldap"]["bind_dn"].as_str().unwrap();
    let bind_pw = yaml["muldap"]["bind_pw"].as_str().unwrap();
    ldap_conn.simple_bind(bind_dn, bind_pw);

    if let Some(matches) = matches.subcommand_matches("user") {
      // create HashMap of user info
      let mut ldap_user_info = HashMap::new();
      ldap_user_info.insert("username", matches.value_of("username").unwrap());
      ldap_user_info.insert("email", matches.value_of("email").unwrap());
      ldap_user_info.insert("first-name", matches.value_of("first-name").unwrap());
      ldap_user_info.insert("last-name", matches.value_of("last-name").unwrap());
      ldap_user_info.insert("id-number", matches.value_of("id-number").unwrap());

      let user_conf = &yaml["muldap"]["user"];

      match ldap::do_add_user(ldap_conn, user_conf.to_owned(), &ldap_user_info) {
        Ok(_) => {},
        Err(e) => println!("{:?}", e),
      }
    }
  }
}
